@@ -1,7 +1,7 @@
-// Copyright (c) 2014, Facebook, Inc. All rights reserved.
-// This source code is licensed under the BSD-style license found in the
-// LICENSE file in the root directory of this source tree. An additional grant
-// of patent rights can be found in the PATENTS file in the same directory.
+// Copyright (c) 2011-present, Facebook, Inc. All rights reserved.
+//  This source code is licensed under both the GPLv2 (found in the
+//  COPYING file in the root directory) and Apache 2.0 License
+//  (found in the LICENSE.Apache file in the root directory).
 
 #ifndef ROCKSDB_LITE
 
@@ -49,12 +49,16 @@ class CuckooBuilderTest : public testing::Test {
     uint64_t read_file_size;
     ASSERT_OK(env_->GetFileSize(fname, &read_file_size));
 
+	  Options options;
+	  options.allow_mmap_reads = true;
+	  ImmutableCFOptions ioptions(options);
+
     // Assert Table Properties.
     TableProperties* props = nullptr;
     unique_ptr<RandomAccessFileReader> file_reader(
-        new RandomAccessFileReader(std::move(read_file)));
+        new RandomAccessFileReader(std::move(read_file), fname));
     ASSERT_OK(ReadTableProperties(file_reader.get(), read_file_size,
-                                  kCuckooTableMagicNumber, env_, nullptr,
+                                  kCuckooTableMagicNumber, ioptions,
                                   &props));
     // Check unused bucket.
     std::string unused_key = props->user_collected_properties[
@@ -62,8 +66,8 @@ class CuckooBuilderTest : public testing::Test {
     ASSERT_EQ(expected_unused_bucket.substr(0,
           props->fixed_key_len), unused_key);
 
-    uint32_t value_len_found =
-      *reinterpret_cast<const uint32_t*>(props->user_collected_properties[
+    uint64_t value_len_found =
+      *reinterpret_cast<const uint64_t*>(props->user_collected_properties[
                 CuckooTablePropertyNames::kValueLength].data());
     ASSERT_EQ(values.empty() ? 0 : values[0].size(), value_len_found);
     ASSERT_EQ(props->raw_value_size, values.size()*value_len_found);
@@ -89,6 +93,8 @@ class CuckooBuilderTest : public testing::Test {
     ASSERT_EQ(props->data_size, expected_unused_bucket.size() *
         (expected_table_size + expected_cuckoo_block_size - 1));
     ASSERT_EQ(props->raw_key_size, keys.size()*props->fixed_key_len);
+    ASSERT_EQ(props->column_family_id, 0);
+    ASSERT_EQ(props->column_family_name, kDefaultColumnFamilyName);
     delete props;
 
     // Check contents of the bucket.
@@ -102,7 +108,7 @@ class CuckooBuilderTest : public testing::Test {
           std::find(expected_locations.begin(), expected_locations.end(), i) -
           expected_locations.begin();
       if (key_idx == keys.size()) {
-        // i is not one of the expected locaitons. Empty bucket.
+        // i is not one of the expected locations. Empty bucket.
         ASSERT_EQ(read_slice.compare(expected_unused_bucket), 0);
       } else {
         keys_found[key_idx] = true;
@@ -118,7 +124,7 @@ class CuckooBuilderTest : public testing::Test {
   std::string GetInternalKey(Slice user_key, bool zero_seqno) {
     IterKey ikey;
     ikey.SetInternalKey(user_key, zero_seqno ? 0 : 1000, kTypeValue);
-    return ikey.GetKey().ToString();
+    return ikey.GetInternalKey().ToString();
   }
 
   uint64_t NextPowOf2(uint64_t num) {
@@ -148,7 +154,8 @@ TEST_F(CuckooBuilderTest, SuccessWithEmptyFile) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, 4, 100,
                              BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   ASSERT_EQ(0UL, builder.FileSize());
   ASSERT_OK(builder.Finish());
@@ -183,7 +190,8 @@ TEST_F(CuckooBuilderTest, WriteSuccessNoCollisionFullKey) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(keys[i]), Slice(values[i]));
@@ -230,7 +238,8 @@ TEST_F(CuckooBuilderTest, WriteSuccessWithCollisionFullKey) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(keys[i]), Slice(values[i]));
@@ -276,9 +285,10 @@ TEST_F(CuckooBuilderTest, WriteSuccessWithCollisionAndCuckooBlock) {
   ASSERT_OK(env_->NewWritableFile(fname, &writable_file, env_options_));
   unique_ptr<WritableFileWriter> file_writer(
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
-  CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
-                             100, BytewiseComparator(), cuckoo_block_size,
-                             false, false, GetSliceHash);
+  CuckooTableBuilder builder(
+      file_writer.get(), kHashTableRatio, num_hash_fun, 100,
+      BytewiseComparator(), cuckoo_block_size, false, false, GetSliceHash,
+      0 /* column_family_id */, kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(keys[i]), Slice(values[i]));
@@ -330,7 +340,8 @@ TEST_F(CuckooBuilderTest, WithCollisionPathFullKey) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(keys[i]), Slice(values[i]));
@@ -379,7 +390,8 @@ TEST_F(CuckooBuilderTest, WithCollisionPathFullKeyAndCuckooBlock) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 2, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(keys[i]), Slice(values[i]));
@@ -421,7 +433,8 @@ TEST_F(CuckooBuilderTest, WriteSuccessNoCollisionUserKey) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(GetInternalKey(user_keys[i], true)), Slice(values[i]));
@@ -464,7 +477,8 @@ TEST_F(CuckooBuilderTest, WriteSuccessWithCollisionUserKey) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(GetInternalKey(user_keys[i], true)), Slice(values[i]));
@@ -509,7 +523,8 @@ TEST_F(CuckooBuilderTest, WithCollisionPathUserKey) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              2, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(GetInternalKey(user_keys[i], true)), Slice(values[i]));
@@ -553,7 +568,8 @@ TEST_F(CuckooBuilderTest, FailWhenCollisionPathTooLong) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              2, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
   for (uint32_t i = 0; i < user_keys.size(); i++) {
     builder.Add(Slice(GetInternalKey(user_keys[i], false)), Slice("value"));
@@ -580,7 +596,8 @@ TEST_F(CuckooBuilderTest, FailWhenSameKeyInserted) {
       new WritableFileWriter(std::move(writable_file), EnvOptions()));
   CuckooTableBuilder builder(file_writer.get(), kHashTableRatio, num_hash_fun,
                              100, BytewiseComparator(), 1, false, false,
-                             GetSliceHash);
+                             GetSliceHash, 0 /* column_family_id */,
+                             kDefaultColumnFamilyName);
   ASSERT_OK(builder.status());
 
   builder.Add(Slice(GetInternalKey(user_key, false)), Slice("value1"));
